--[[
    Exemple: using a naive Bayes classifier to sort text messages into
    'spam' or 'ham'. Uses the dataset found at [1], that is a
    'bag of words', built from the processed text data.
    
    I've found that, for this dataset, it is better to ignore multiple
    occurences of the same word.
    
    [1]: http://openclassroom.stanford.edu/MainFolder/DocumentPage.php?
         course=MachineLearning&doc=exercises/ex6/ex6.html
--]]
require('lna.lab')

-- ** training **

-- load the feature (X) and label (y) matrices
local X = matrix.load("train-features", " ")
local y = matrix.load("train-labels"  , " ")

-- separate in the 'ham' and 'spam' groups
local ham  = X:filter(function(_,i) return y(i) == 0 end):clip(0,1,true)
local spam = X:filter(function(_,i) return y(i) == 1 end):clip(0,1,true)

-- compute the values of P(Fi | C), for every feature Fi and class C
-- P(Fi | C) will be stored on column i
local dict_wc, ham_wc, spam_wc  = ham.n, ham:sum(), spam:sum()

-- could be implemented with foreach() to reduce memory usage
local prob_ham  = (ham:summ() + 1):div(dict_wc + ham_wc ):t()
local prob_spam = (spam:summ()+ 1):div(dict_wc + spam_wc):t()

-- ** test **

-- load the test feature and label matrices
local Xt = matrix.load("test-features", " "):clip(0,1,true)
local yt = matrix.load("test-labels"  , " "):clip(0,1,true)

-- Uses log() to avoid precision errors and to convert and to
-- matemagically skip missing words that are not present in the message
local res_ham  = Xt * prob_ham:log()
local res_spam = Xt * prob_spam:log()
local is_spam  = res_spam:gt(res_ham)

local errors = is_spam:xor(yt):sum()
print(string.format('Errors: %d (%.2f%%)', errors, (errors / Xt.m) * 100))
